/** @module : PiSo
 *  @author : Adaptive & Secure Computing Systems (ASCS) Laboratory

 *  Copyright (c) 2018 ABAQS (ASCS/ECE/BU)
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.

 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */


// Parallel in Serial out shift register
module PiSo #(
  parameter BIT_WIDTH = 32,
  parameter DEPTH     = 4
) (
  input clock,
  input reset_n,
  input shift,
  input [(DEPTH*BIT_WIDTH)-1:0] data_in,
  output [BIT_WIDTH-1:0] out
);

reg [BIT_WIDTH-1:0] shift_reg [DEPTH-1:0];

assign out = shift_reg[DEPTH-1];

genvar i;
generate
for( i=1; i<DEPTH; i=i+1) begin : sr_loop
  always@(posedge clock or negedge reset_n) begin
  	if(reset_n ==1'b0) begin
			shift_reg[i] <= {BIT_WIDTH{1'b0}};
		end else if ( shift ) begin
			shift_reg[i] <= shift_reg[i-1];
		end else begin
			shift_reg[i] <= data_in[(BIT_WIDTH*(i+1)-1):BIT_WIDTH*i];
		end
  end // always
end // for sr_loop
endgenerate

always@(posedge clock or negedge reset_n) begin
  if(reset_n == 1'b0) begin
    shift_reg[0] <= {BIT_WIDTH{1'b0}};
  end else begin
    shift_reg[0] <= data_in[BIT_WIDTH-1:0];
  end
end // sr[0] always

endmodule
