/** @module : SiPo
 *  @author : Adaptive & Secure Computing Systems (ASCS) Laboratory

 *  Copyright (c) 2018 ABAQS (ASCS/ECE/BU)
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.

 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 */

// Serial in Parallel out shift register
module SiPo #(
  parameter BIT_WIDTH = 32,
  parameter DEPTH     = 4
) (
  input clock,
  input reset_n,
  input shift,
  input [BIT_WIDTH-1:0] in,
  output [(DEPTH*BIT_WIDTH)-1:0] data_out
);

reg [BIT_WIDTH-1:0] shift_reg [DEPTH-1:0];

genvar i;
generate
for( i=0; i<DEPTH; i=i+1) begin : out_loop
  // Connect register to outputs
  assign data_out[(BIT_WIDTH*(i+1))-1:BIT_WIDTH*i] = shift_reg[i];
end // for

for( i=1; i<DEPTH; i=i+1) begin : sr_loop
  // Connect Shift register
  always@(posedge clock or negedge reset_n) begin
  	if(reset_n == 1'b0) begin
			shift_reg[i] <= {BIT_WIDTH{1'b0}};
		end else if ( shift ) begin
			shift_reg[i] <= shift_reg[i-1];
		end else begin
      shift_reg[i] <= shift_reg[i];
		end
  end // always
end // for sr_loop

endgenerate


always@(posedge clock or negedge reset_n) begin
  if(reset_n == 1'b0) begin
    shift_reg[0] <= {BIT_WIDTH{1'b0}};
  end else if( shift ) begin
    shift_reg[0] <= in;
  end else begin
    shift_reg[0] <= shift_reg[0];
  end
end // sr[0] always

endmodule
